@@ -12,10 +12,10 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import re
-
+import jsonpatch
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
+import six
 from sqlalchemy import desc
 from webob import exc
 
@@ -39,19 +39,22 @@ LOG = logging.getLogger(__name__)
 
 API_NAME = 'Environments'
 
-VALID_NAME_REGEX = re.compile('^[a-zA-Z]+[\w.-]*$')
-
 
 class Controller(object):
     @request_statistics.stats_count(API_NAME, 'Index')
     def index(self, request):
         all_tenants = request.GET.get('all_tenants', 'false').lower() == 'true'
-        LOG.debug('Environments:List <all_tenants: {tenants}>'.format(
-                  tenants=all_tenants))
+        tenant = request.GET.get('tenant', None)
+        LOG.debug('Environments:List <all_tenants: {tenants}, '
+                  'tenant: {tenant}>'.format(tenants=all_tenants,
+                                             tenant=tenant))
 
         if all_tenants:
             policy.check('list_environments_all_tenants', request.context)
             filters = {}
+        elif tenant:
+            policy.check('list_environments_all_tenants', request.context)
+            filters = {'tenant_id': tenant}
         else:
             policy.check('list_environments', request.context)
             # Only environments from same tenant as user should be returned
@@ -67,30 +70,24 @@ class Controller(object):
         LOG.debug('Environments:Create <Body {body}>'.format(body=body))
         policy.check('create_environment', request.context)
 
-        if not body.get('name'):
+        if not('name' in body and body['name'].strip()):
             msg = _('Please, specify a name of the environment to create')
-            LOG.exception(msg)
+            LOG.error(msg)
             raise exc.HTTPBadRequest(explanation=msg)
 
-        name = unicode(body['name'])
+        name = six.text_type(body['name'])
         if len(name) > 255:
             msg = _('Environment name should be 255 characters maximum')
-            LOG.exception(msg)
+            LOG.error(msg)
             raise exc.HTTPBadRequest(explanation=msg)
-        if VALID_NAME_REGEX.match(name):
-            try:
-                environment = envs.EnvironmentServices.create(
-                    body.copy(),
-                    request.context)
-            except db_exc.DBDuplicateEntry:
-                msg = _('Environment with specified name already exists')
-                LOG.exception(msg)
-                raise exc.HTTPConflict(explanation=msg)
-        else:
-            msg = _('Environment name must contain only alphanumeric or "_-." '
-                    'characters, must start with alpha')
-            LOG.exception(msg)
-            raise exc.HTTPClientError(explanation=msg)
+        try:
+            environment = envs.EnvironmentServices.create(
+                body.copy(),
+                request.context)
+        except db_exc.DBDuplicateEntry:
+            msg = _('Environment with specified name already exists')
+            LOG.error(msg)
+            raise exc.HTTPConflict(explanation=msg)
 
         return environment.to_dict()
 
@@ -138,7 +135,12 @@ class Controller(object):
 
         session = db_session.get_session()
         environment = session.query(models.Environment).get(environment_id)
-        if VALID_NAME_REGEX.match(str(body['name'])):
+        new_name = six.text_type(body['name'])
+        if new_name.strip():
+            if len(new_name) > 255:
+                msg = _('Environment name should be 255 characters maximum')
+                LOG.error(msg)
+                raise exc.HTTPBadRequest(explanation=msg)
             try:
                 environment.update(body)
                 environment.save(session)
@@ -147,8 +149,8 @@ class Controller(object):
                 LOG.error(msg)
                 raise exc.HTTPConflict(explanation=msg)
         else:
-            msg = _('Environment name must contain only alphanumeric '
-                    'or "_-." characters, must start with alpha')
+            msg = _('Environment name must contain at least one '
+                    'non-white space symbol')
             LOG.error(msg)
             raise exc.HTTPClientError(explanation=msg)
 
@@ -196,6 +198,62 @@ class Controller(object):
                 result[service_id] = None
         return {'lastStatuses': result}
 
+    @request_statistics.stats_count(API_NAME, 'GetModel')
+    @verify_env
+    def get_model(self, request, environment_id, path):
+        LOG.debug('Environments:GetModel <Id: %(env_id)s>, Path: %(path)s',
+                  {'env_id': environment_id, 'path': path})
+        target = {"environment_id": environment_id}
+        policy.check('show_environment', request.context, target)
+
+        session_id = None
+        if hasattr(request, 'context') and request.context.session:
+            session_id = request.context.session
+
+        get_description = envs.EnvironmentServices.get_environment_description
+        env_model = get_description(environment_id, session_id)
+        try:
+            result = utils.TraverseHelper.get(path, env_model)
+        except (KeyError, ValueError):
+            raise exc.HTTPNotFound
+
+        return result
+
+    @request_statistics.stats_count(API_NAME, 'UpdateModel')
+    @verify_env
+    def update_model(self, request, environment_id, body=None):
+        if not body:
+            msg = _('Request body is empty: please, provide '
+                    'environment object model patch')
+            LOG.error(msg)
+            raise exc.HTTPBadRequest(msg)
+        LOG.debug('Environments:UpdateModel <Id: %(env_id)s, Body: %(body)s>',
+                  {'env_id': environment_id, 'body': body})
+        target = {"environment_id": environment_id}
+        policy.check('update_environment', request.context, target)
+
+        session_id = None
+        if hasattr(request, 'context') and request.context.session:
+            session_id = request.context.session
+
+        get_description = envs.EnvironmentServices.get_environment_description
+        env_model = get_description(environment_id, session_id)
+
+        for change in body:
+            change['path'] = '/' + '/'.join(change['path'])
+
+        patch = jsonpatch.JsonPatch(body)
+        try:
+            patch.apply(env_model, in_place=True)
+        except jsonpatch.JsonPatchException as e:
+            raise exc.HTTPNotFound(str(e))
+
+        save_description = envs.EnvironmentServices. \
+            save_environment_description
+        save_description(session_id, env_model)
+
+        return env_model
+
 
 def create_resource():
     return wsgi.Resource(Controller())
