@@ -1,4 +1,4 @@
-# Copyright 2011 OpenStack LLC.
+# Copyright 2011 OpenStack Foundation
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,14 +14,12 @@
 #    under the License.
 
 from oslo_config import cfg
-from oslo_config import types
 from oslo_log import log as logging
+from oslo_middleware import cors
 
 from murano.common.i18n import _
 from murano import version
 
-portType = types.Integer(1, 65535)
-
 paste_deploy_opts = [
     cfg.StrOpt('flavor', help='Paste flavor'),
     cfg.StrOpt('config_file', help='Path to Paste config file'),
@@ -30,10 +28,9 @@ paste_deploy_opts = [
 bind_opts = [
     cfg.StrOpt('bind-host', default='0.0.0.0',
                help='Address to bind the Murano API server to.'),
-    cfg.Opt('bind-port',
-            type=portType,
-            default=8082,
-            help='Port the bind the Murano API server to.'),
+    cfg.PortOpt('bind-port',
+                default=8082,
+                help='Port the bind the Murano API server to.'),
 ]
 
 rabbit_opts = [
@@ -41,15 +38,14 @@ rabbit_opts = [
                help='The RabbitMQ broker address which used for communication '
                'with Murano guest agents.'),
 
-    cfg.Opt('port',
-            type=portType,
-            default=5672,
-            help='The RabbitMQ broker port.'),
+    cfg.PortOpt('port',
+                default=5672,
+                help='The RabbitMQ broker port.'),
 
     cfg.StrOpt('login', default='guest',
                help='The RabbitMQ login.'),
 
-    cfg.StrOpt('password', default='guest',
+    cfg.StrOpt('password', default='guest', secret=True,
                help='The RabbitMQ password.'),
 
     cfg.StrOpt('virtual_host', default='/',
@@ -58,11 +54,18 @@ rabbit_opts = [
     cfg.BoolOpt('ssl', default=False,
                 help='Boolean flag to enable SSL communication through the '
                 'RabbitMQ broker between murano-engine and guest agents.'),
+
     cfg.StrOpt('ca_certs', default='',
-               help='SSL cert file (valid only if SSL enabled).')
+               help='SSL cert file (valid only if SSL enabled).'),
+
+    cfg.BoolOpt('insecure', default=False,
+                help='This option explicitly allows Murano to perform '
+                     '"insecure" SSL connections to RabbitMQ'),
 ]
 
 heat_opts = [
+    cfg.StrOpt('url', help='Optional heat endpoint override'),
+
     cfg.BoolOpt('insecure', default=False,
                 help='This option explicitly allows Murano to perform '
                 '"insecure" SSL connections and transfers with Heat API.'),
@@ -79,17 +82,34 @@ heat_opts = [
                                 'communicate with Heat API.'),
 
     cfg.StrOpt('endpoint_type', default='publicURL',
-               help='Heat endpoint type.')
+               help='Heat endpoint type.'),
+
+    cfg.ListOpt('stack_tags', default=['murano'],
+                help='List of tags to be assigned to heat stacks created '
+                     'during environment deployment.')
 ]
 
 mistral_opts = [
+    cfg.StrOpt('url', help='Optional mistral endpoint override'),
+
     cfg.StrOpt('endpoint_type', default='publicURL',
                help='Mistral endpoint type.'),
+
     cfg.StrOpt('service_type', default='workflowv2',
-               help='Mistral service type.')
+               help='Mistral service type.'),
+
+    cfg.BoolOpt('insecure', default=False,
+                help='This option explicitly allows Murano to perform '
+                '"insecure" SSL connections and transfers with Mistral.'),
+
+    cfg.StrOpt('ca_cert',
+               help='(SSL) Tells Murano to use the specified client '
+               'certificate file when communicating with Mistral.')
 ]
 
 neutron_opts = [
+    cfg.StrOpt('url', help='Optional neutron endpoint override'),
+
     cfg.BoolOpt('insecure', default=False,
                 help='This option explicitly allows Murano to perform '
                 '"insecure" SSL connections and transfers with Neutron API.'),
@@ -102,24 +122,6 @@ neutron_opts = [
                help='Neutron endpoint type.')
 ]
 
-keystone_opts = [
-    cfg.BoolOpt('insecure', default=False,
-                help='This option explicitly allows Murano to perform '
-                     '"insecure" SSL connections and transfers with '
-                     'Keystone API running Kyestone API.'),
-
-    cfg.StrOpt('ca_file',
-               help='(SSL) Tells Murano to use the specified certificate file '
-                    'to verify the peer when communicating with Keystone.'),
-
-    cfg.StrOpt('cert_file',
-               help='(SSL) Tells Murano to use the specified client '
-                    'certificate file when communicating with Keystone.'),
-
-    cfg.StrOpt('key_file', help='(SSL/SSH) Private key file name to '
-                                'communicate with Keystone API')
-]
-
 murano_opts = [
     cfg.StrOpt('url', help='Optional murano url in format '
                            'like http://0.0.0.0:8082 used by Murano engine'),
@@ -146,13 +148,27 @@ murano_opts = [
     cfg.StrOpt('endpoint_type', default='publicURL',
                help='Murano endpoint type used by Murano engine.'),
 
-    cfg.ListOpt('enabled_plugins', default=None,
+    cfg.ListOpt('enabled_plugins',
                 help="List of enabled Extension Plugins. "
                      "Remove or leave commented to enable all installed "
                      "plugins."),
 
-    cfg.StrOpt('region_name_for_services',
-               help="Default region name used to get services endpoints.")
+    cfg.IntOpt('package_size_limit', default=5,
+               help='Maximum application package size, Mb',
+               deprecated_group='packages_opts'),
+
+    cfg.IntOpt('limit_param_default', default=20,
+               help='Default value for package pagination in API.',
+               deprecated_group='packages_opts'),
+
+    cfg.IntOpt('api_limit_max', default=100,
+               help='Maximum number of packages to be returned in a single '
+                    'pagination request',
+               deprecated_group='packages_opts'),
+
+    cfg.IntOpt('api_workers',
+               help=_('Number of API workers')),
+
 ]
 
 networking_opts = [
@@ -185,8 +201,13 @@ networking_opts = [
 
     cfg.StrOpt('network_config_file', default='netconfig.yaml',
                help='If provided networking configuration will be taken '
-                    'from this file')
+                    'from this file'),
+
+    cfg.StrOpt('driver', choices=['neutron', 'nova'],
+               help='Network driver to use. Options are neutron or nova.'
+                    'If not provided, the driver will be detected.'),
 ]
+
 stats_opts = [
     cfg.IntOpt('period', default=5,
                help=_('Statistics collection interval in minutes.'
@@ -206,9 +227,32 @@ engine_opts = [
     cfg.IntOpt('agent_timeout', default=3600,
                help=_('Time for waiting for a response from murano agent '
                       'during the deployment')),
+    cfg.IntOpt('engine_workers',
+               deprecated_opts=[cfg.DeprecatedOpt('workers',
+                                                  group='engine')],
+               help=_('Number of engine workers')),
+
     cfg.ListOpt('load_packages_from', default=[],
                 help=_('List of directories to load local packages from. '
-                       'If not provided, packages will be loaded by API'))
+                       'If not provided, packages will be loaded only API'),
+                deprecated_group='packages_opts'),
+
+    cfg.StrOpt('packages_cache',
+               help='Location (directory) for Murano package cache.',
+               deprecated_group='packages_opts'),
+
+    cfg.BoolOpt('enable_packages_cache', default=True,
+                help=_('Enables murano-engine to persist on disk '
+                       'packages downloaded during deployments. '
+                       'The packages would be re-used for consequent '
+                       'deployments.'),
+                deprecated_group='packages_opts'),
+
+    cfg.StrOpt('packages_service', default='murano',
+               help=_('The service to store murano packages: murano (stands '
+                      'for legacy behavior using murano-api) or glance '
+                      '(stands for glance-glare artifact service)'),
+               deprecated_group='packages_opts'),
 ]
 
 # TODO(sjmc7): move into engine opts?
@@ -217,29 +261,37 @@ metadata_dir = [
                help='Metadata dir')
 ]
 
-packages_opts = [
-    cfg.StrOpt('packages_cache', default=None,
-               help='Location (directory) for Murano package cache.'),
+glare_opts = [
+    cfg.StrOpt('url', help='Optional glare url in format '
+                           'like http://0.0.0.0:9494 used by Glare API',
+               deprecated_group='glance'),
 
-    cfg.IntOpt('package_size_limit', default=5,
-               help='Maximum application package size, Mb'),
+    cfg.BoolOpt('insecure', default=False,
+                help='This option explicitly allows Murano to perform '
+                '"insecure" SSL connections and transfers with Glare API.',
+                deprecated_group='glance'),
 
-    cfg.IntOpt('limit_param_default', default=20,
-               help='Default value for package pagination in API.'),
+    cfg.StrOpt('ca_file',
+               help='(SSL) Tells Murano to use the specified certificate file '
+               'to verify the peer running Glare API.',
+               deprecated_group='glance'),
 
-    cfg.IntOpt('api_limit_max', default=100,
-               help='Maximum number of packages to be returned in a single '
-                    'pagination request'),
+    cfg.StrOpt('cert_file',
+               help='(SSL) Tells Murano to use the specified client '
+               'certificate file when communicating with Glare.',
+               deprecated_group='glance'),
 
-    cfg.StrOpt('packages_service', default='murano',
-               help=_('The service to store murano packages: murano (stands '
-                      'for legacy behavior using murano-api) or glance '
-                      '(stands for Glance V3 artifact repository)'))
+    cfg.StrOpt('key_file', help='(SSL/SSH) Private key file name to '
+                                'communicate with Glare API.',
+               deprecated_group='glance'),
+
+    cfg.StrOpt('endpoint_type', default='publicURL',
+               help='Glare endpoint type.',
+               deprecated_group='glance')
 ]
 
 glance_opts = [
-    cfg.StrOpt('url', help='Optional murano url in format '
-                           'like http://0.0.0.0:9292 used by Glance API'),
+    cfg.StrOpt('url', help='Optional glance endpoint override'),
 
     cfg.BoolOpt('insecure', default=False,
                 help='This option explicitly allows Murano to perform '
@@ -247,11 +299,11 @@ glance_opts = [
 
     cfg.StrOpt('ca_file',
                help='(SSL) Tells Murano to use the specified certificate file '
-               'to verify the peer running Glance API.'),
+                    'to verify the peer running Glance API.'),
 
     cfg.StrOpt('cert_file',
                help='(SSL) Tells Murano to use the specified client '
-               'certificate file when communicating with Glance.'),
+                    'certificate file when communicating with Glance.'),
 
     cfg.StrOpt('key_file', help='(SSL/SSH) Private key file name to '
                                 'communicate with Glance API.'),
@@ -265,6 +317,11 @@ file_server = [
                help='Set a file server.')
 ]
 
+home_region = cfg.StrOpt(
+    'home_region',
+    help="Default region name used to get services endpoints.")
+
+
 CONF = cfg.CONF
 CONF.register_opts(paste_deploy_opts, group='paste_deploy')
 CONF.register_cli_opts(bind_opts)
@@ -272,22 +329,46 @@ CONF.register_opts(rabbit_opts, group='rabbitmq')
 CONF.register_opts(heat_opts, group='heat')
 CONF.register_opts(mistral_opts, group='mistral')
 CONF.register_opts(neutron_opts, group='neutron')
-CONF.register_opts(keystone_opts, group='keystone')
 CONF.register_opts(murano_opts, group='murano')
 CONF.register_opts(engine_opts, group='engine')
 CONF.register_opts(file_server)
+CONF.register_opt(home_region)
 CONF.register_cli_opts(metadata_dir)
-CONF.register_opts(packages_opts, group='packages_opts')
 CONF.register_opts(stats_opts, group='stats')
 CONF.register_opts(networking_opts, group='networking')
+CONF.register_opts(glare_opts, group='glare')
 CONF.register_opts(glance_opts, group='glance')
 
 
 def parse_args(args=None, usage=None, default_config_files=None):
     logging.register_options(CONF)
-    logging.setup(CONF, 'murano')
     CONF(args=args,
          project='murano',
          version=version.version_string,
          usage=usage,
          default_config_files=default_config_files)
+
+
+def set_middleware_defaults():
+    """Update default configuration options for oslo.middleware."""
+    # CORS Defaults
+    # TODO(krotscheck): Update with https://review.openstack.org/#/c/285368/
+    cfg.set_defaults(cors.CORS_OPTS,
+                     allow_headers=['X-Auth-Token',
+                                    'X-Openstack-Request-Id',
+                                    'X-Configuration-Session',
+                                    'X-Roles',
+                                    'X-User-Id',
+                                    'X-Tenant-Id'],
+                     expose_headers=['X-Auth-Token',
+                                     'X-Openstack-Request-Id',
+                                     'X-Configuration-Session',
+                                     'X-Roles',
+                                     'X-User-Id',
+                                     'X-Tenant-Id'],
+                     allow_methods=['GET',
+                                    'PUT',
+                                    'POST',
+                                    'DELETE',
+                                    'PATCH']
+                     )
