@@ -13,16 +13,16 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import base64
 import collections
 import random
 import re
 import string
 import time
-import types
 
 import jsonpatch
 import jsonpointer
+from oslo_serialization import base64
+import six
 from yaql.language import specs
 from yaql.language import utils
 from yaql.language import yaqltypes
@@ -40,13 +40,13 @@ _random_string_counter = None
 @specs.parameter('value', yaqltypes.String())
 @specs.extension_method
 def base64encode(value):
-    return base64.b64encode(value)
+    return base64.encode_as_text(value)
 
 
 @specs.parameter('value', yaqltypes.String())
 @specs.extension_method
 def base64decode(value):
-    return base64.b64decode(value)
+    return base64.decode_as_text(value)
 
 
 @specs.parameter('collection', yaqltypes.Iterable())
@@ -59,7 +59,7 @@ def pselect(collection, composer):
 @specs.parameter('mappings', collections.Mapping)
 @specs.extension_method
 def bind(obj, mappings):
-    if isinstance(obj, types.StringTypes) and obj.startswith('$'):
+    if isinstance(obj, six.string_types) and obj.startswith('$'):
         value = _convert_macro_parameter(obj[1:], mappings)
         if value is not None:
             return value
@@ -67,10 +67,10 @@ def bind(obj, mappings):
         return [bind(t, mappings) for t in obj]
     elif isinstance(obj, collections.Mapping):
         result = {}
-        for key, value in obj.iteritems():
+        for key, value in obj.items():
             result[bind(key, mappings)] = bind(value, mappings)
         return result
-    elif isinstance(obj, types.StringTypes) and obj.startswith('$'):
+    elif isinstance(obj, six.string_types) and obj.startswith('$'):
         value = _convert_macro_parameter(obj[1:], mappings)
         if value is not None:
             return value
@@ -82,7 +82,7 @@ def _convert_macro_parameter(macro, mappings):
 
     def replace(match):
         replaced[0] = True
-        return unicode(mappings.get(match.group(1)))
+        return six.text_type(mappings.get(match.group(1)))
 
     result = re.sub('{(\\w+?)}', replace, macro)
     if replaced[0]:
@@ -93,8 +93,14 @@ def _convert_macro_parameter(macro, mappings):
 
 @specs.parameter('group', yaqltypes.String())
 @specs.parameter('setting', yaqltypes.String())
-def config(group, setting):
-    return cfg.CONF[group][setting]
+@specs.parameter('read_as_file', bool)
+def config(group, setting, read_as_file=False):
+    config_value = cfg.CONF[group][setting]
+    if read_as_file:
+        with open(config_value) as target_file:
+            return target_file.read()
+    else:
+        return config_value
 
 
 @specs.parameter('setting', yaqltypes.String())
@@ -127,13 +133,15 @@ def patch_(engine, obj, patch):
 
 def _int2base(x, base):
     """Converts decimal integers into another number base
-     from base-2 to base-36.
+
+    Converts decimal integers into another number base
+    from base-2 to base-36.
 
     :param x: decimal integer
     :param base: number base, max value is 36
     :return: integer converted to the specified base
     """
-    digs = string.digits + string.lowercase
+    digs = string.digits + string.ascii_lowercase
     if x < 0:
         sign = -1
     elif x == 0:
@@ -144,7 +152,7 @@ def _int2base(x, base):
     digits = []
     while x:
         digits.append(digs[x % base])
-        x /= base
+        x //= base
     if sign < 0:
         digits.append('-')
     digits.reverse()
@@ -152,8 +160,10 @@ def _int2base(x, base):
 
 
 def random_name():
-    """Replace '#' char in pattern with supplied number, if no pattern is
-       supplied generate short and unique name for the host.
+    """Replace '#' char in pattern with supplied number
+
+    Replace '#' char in pattern with supplied number. If no pattern is
+    supplied, generate a short and unique name for the host.
 
     :param pattern: hostname pattern
     :param number: number to replace with in pattern
@@ -163,7 +173,7 @@ def random_name():
 
     counter = _random_string_counter or 1
     # generate first 5 random chars
-    prefix = ''.join(random.choice(string.lowercase) for _ in range(5))
+    prefix = ''.join(random.choice(string.ascii_lowercase) for _ in range(5))
     # convert timestamp to higher base to shorten hostname string
     # (up to 8 chars)
     timestamp = _int2base(int(time.time() * 1000), 36)[:8]
@@ -179,7 +189,7 @@ def random_name():
 @specs.extension_method
 def first_or_default(collection, default=None):
     try:
-        return iter(collection).next()
+        return next(iter(collection))
     except StopIteration:
         return default
 
@@ -202,8 +212,6 @@ def get_context(runtime_version):
     context.register_function(bind)
     context.register_function(random_name)
     context.register_function(patch_)
-    context.register_function(config)
-    context.register_function(config_default)
     context.register_function(logger)
 
     if runtime_version <= constants.RUNTIME_VERSION_1_1:
@@ -217,3 +225,11 @@ def get_context(runtime_version):
             for spec in utils.to_extension_method(t, root_context):
                 context.register_function(spec)
     return context
+
+
+@helpers.memoize
+def get_restricted_context():
+    context = yaql_integration.create_empty_context()
+    context.register_function(config)
+    context.register_function(config_default)
+    return context
