@@ -16,22 +16,27 @@ import contextlib
 import json
 import os
 import random
+import re
 import socket
 import telnetlib
 import time
 
 from heatclient import client as heatclient
 from keystoneclient import exceptions as ks_exceptions
-from keystoneclient.v2_0 import client as ksclient
+import keystoneclient.v2_0 as keystoneclientv2
+import keystoneclient.v3 as keystoneclientv3
 from muranoclient import client as mclient
 import muranoclient.common.exceptions as exceptions
+from muranoclient.glance import client as glare_client
 from oslo_log import log as logging
 import yaml
 
+from murano.common.i18n import _LW, _LE
 from murano.services import states
 import murano.tests.functional.common.zip_utils_mixin as zip_utils
 import murano.tests.functional.engine.config as cfg
 
+
 CONF = cfg.cfg.CONF
 
 LOG = logging.getLogger(__name__)
@@ -46,7 +51,9 @@ def ignored(*exceptions):
 
 
 def memoize(f):
-    """Decorator, which saves result of a decorated function
+    """Saves result of decorated function to cache
+
+    Decorator, which saves result of a decorated function
     to cache.
     TTL for cache is 1800 sec
 
@@ -76,10 +83,16 @@ class DeployTestMixin(zip_utils.ZipUtilsMixin):
     @staticmethod
     @memoize
     def keystone_client():
+        region = CONF.murano.region_name
+        if re.match(".*/v3/?$", CONF.murano.auth_url):
+            ksclient = keystoneclientv3
+        else:
+            ksclient = keystoneclientv2
         return ksclient.Client(username=CONF.murano.user,
                                password=CONF.murano.password,
                                tenant_name=CONF.murano.tenant,
-                               auth_url=CONF.murano.auth_url)
+                               auth_url=CONF.murano.auth_url,
+                               region_name=region)
 
     @classmethod
     @memoize
@@ -94,7 +107,17 @@ class DeployTestMixin(zip_utils.ZipUtilsMixin):
     @memoize
     def murano_client(cls):
         murano_url = cls.get_murano_url()
+        if CONF.murano.packages_service == "glare":
+            glare_endpoint = "http://127.0.0.1:9494"
+            artifacts_client = glare_client.Client(
+                endpoint=glare_endpoint,
+                token=cls.keystone_client().auth_token,
+                insecure=False, key_file=None, ca_file=None, cert_file=None,
+                type_name="murano", type_version=1)
+        else:
+            artifacts_client = None
         return mclient.Client('1',
+                              artifacts_client=artifacts_client,
                               endpoint=murano_url,
                               token=cls.keystone_client().auth_token)
 
@@ -121,8 +144,8 @@ class DeployTestMixin(zip_utils.ZipUtilsMixin):
         cls.murano_client().sessions.deploy(environment.id, session.id)
         return environment
 
-    @staticmethod
-    def wait_for_final_status(environment, timeout=300):
+    @classmethod
+    def wait_for_final_status(cls, environment, timeout=300):
         """Function for wait final status of environment.
 
         :param environment: Murano environment.
@@ -139,9 +162,9 @@ class DeployTestMixin(zip_utils.ZipUtilsMixin):
                 raise RuntimeError(err_msg)
             time.sleep(5)
             status = environment.manager.get(environment.id).status
-        dep = environment.manager.api.deployments.list(environment.id)
-        reports = environment.manager.api.deployments.reports(environment.id,
-                                                              dep[0].id)
+        dep = cls.murano_client().deployments.list(environment.id)
+        reports = cls.murano_client().deployments.reports(environment.id,
+                                                          dep[0].id)
         return status, ", ".join([r.text for r in reports])
 
 # -----------------------------Reports methods---------------------------------
@@ -180,7 +203,7 @@ class DeployTestMixin(zip_utils.ZipUtilsMixin):
         deployment = cls.get_last_deployment(environment)
         try:
             details = deployment.result['result']['details']
-            LOG.warning('Details:\n {details}'.format(details=details))
+            LOG.warning(_LW('Details:\n {details}').format(details=details))
         except Exception as e:
             LOG.error(e)
         report = cls.get_deployment_report(environment, deployment)
@@ -311,10 +334,11 @@ class DeployTestMixin(zip_utils.ZipUtilsMixin):
     def get_murano_url(cls):
         try:
             url = cls.keystone_client().service_catalog.url_for(
-                service_type='application_catalog', endpoint_type='publicURL')
+                service_type='application-catalog', endpoint_type='publicURL')
         except ks_exceptions.EndpointNotFound:
             url = CONF.murano.murano_url
-            LOG.warning("Murano endpoint not found in Keystone. Using CONF.")
+            LOG.warning(_LW("Murano endpoint not found in Keystone. "
+                            "Using CONF."))
         return url if 'v1' not in url else "/".join(
             url.split('/')[:url.split('/').index('v1')])
 
@@ -338,7 +362,7 @@ class DeployTestMixin(zip_utils.ZipUtilsMixin):
                 raise RuntimeError('Resource at {0}:{1} not exist'.
                                    format(ip, port))
         except socket.error as e:
-            LOG.error('Socket Error: {error}'.format(error=e))
+            LOG.error(_LE('Socket Error: {error}').format(error=e))
 
     @classmethod
     def get_ip_by_appname(cls, environment, appname):
@@ -509,7 +533,13 @@ class DeployTestMixin(zip_utils.ZipUtilsMixin):
     def _get_stack(cls, environment_id):
 
         for stack in cls.heat_client().stacks.list():
-            if environment_id in stack.description:
+            stack_description = (
+                cls.heat_client().stacks.get(stack.id).description)
+            if not stack_description:
+                err_msg = ("Stack {0} description is empty".format(stack.id))
+                LOG.error(err_msg)
+                raise RuntimeError(err_msg)
+            if environment_id in stack_description:
                 return stack
 
     @classmethod
